@@ -290,20 +290,17 @@ class TestMarker:
                 True,
             ),
             (
-                "python_version ~= '2.7.0' and (os_name == 'foo' or "
-                "os_name == 'bar')",
+                "python_version ~= '2.7.0' and (os_name == 'foo' or os_name == 'bar')",
                 {"os_name": "foo", "python_version": "2.7.4"},
                 True,
             ),
             (
-                "python_version ~= '2.7.0' and (os_name == 'foo' or "
-                "os_name == 'bar')",
+                "python_version ~= '2.7.0' and (os_name == 'foo' or os_name == 'bar')",
                 {"os_name": "bar", "python_version": "2.7.4"},
                 True,
             ),
             (
-                "python_version ~= '2.7.0' and (os_name == 'foo' or "
-                "os_name == 'bar')",
+                "python_version ~= '2.7.0' and (os_name == 'foo' or os_name == 'bar')",
                 {"os_name": "other", "python_version": "2.7.4"},
                 False,
             ),
@@ -350,8 +347,7 @@ class TestMarker:
                 False,
             ),
             (
-                "python_version == '2.5' and platform.python_implementation"
-                "!= 'Jython'",
+                "python_version == '2.5' and platform.python_implementation!= 'Jython'",
                 {"python_version": "2.7"},
                 False,
             ),
@@ -398,3 +394,35 @@ class TestMarker:
     def test_python_full_version_untagged(self):
         with mock.patch("platform.python_version", return_value="3.11.1+"):
             assert Marker("python_full_version < '3.12'").evaluate()
+
+    @pytest.mark.parametrize("variable", ["extras", "dependency_groups"])
+    @pytest.mark.parametrize(
+        "expression,result",
+        [
+            pytest.param('"foo" in {0}', True, id="value-in-foo"),
+            pytest.param('"bar" in {0}', True, id="value-in-bar"),
+            pytest.param('"baz" in {0}', False, id="value-not-in"),
+            pytest.param('"baz" not in {0}', True, id="value-not-in-negated"),
+            pytest.param('"foo" in {0} and "bar" in {0}', True, id="and-in"),
+            pytest.param('"foo" in {0} or "bar" in {0}', True, id="or-in"),
+            pytest.param(
+                '"baz" in {0} and "foo" in {0}', False, id="short-circuit-and"
+            ),
+            pytest.param('"foo" in {0} or "baz" in {0}', True, id="short-circuit-or"),
+            pytest.param('"Foo" in {0}', True, id="case-sensitive"),
+        ],
+    )
+    def test_extras_and_dependency_groups(self, variable, expression, result):
+        environment = {variable: {"foo", "bar"}}
+        assert Marker(expression.format(variable)).evaluate(environment) == result
+
+    @pytest.mark.parametrize("variable", ["extras", "dependency_groups"])
+    def test_extras_and_dependency_groups_disallowed(self, variable):
+        marker = Marker(f'"foo" in {variable}')
+        assert not marker.evaluate(context="lock_file")
+
+        with pytest.raises(KeyError):
+            marker.evaluate()
+
+        with pytest.raises(KeyError):
+            marker.evaluate(context="requirement")
