@@ -72,7 +72,7 @@ def test_get_musl_version(monkeypatch, executable, output, version, ld_musl):
             pretend.call(
                 [ld_musl],
                 stderr=subprocess.PIPE,
-                universal_newlines=True,
+                text=True,
             )
         ]
     else:
