@@ -2,7 +2,7 @@
 # 2.0, and the BSD License. See the LICENSE file in the root of this repository
 # for complete details.
 
-from typing import Optional, Set
+from __future__ import annotations
 
 import pytest
 
@@ -26,6 +26,10 @@ EQUAL_DEPENDENCIES = [
     ),
 ]
 
+EQUIVALENT_DEPENDENCIES = [
+    ("scikit-learn==1.0.1", "scikit_learn==1.0.1"),
+]
+
 DIFFERENT_DEPENDENCIES = [
     ("package_one", "package_two"),
     ("packaging>20.1", "packaging>=20.1"),
@@ -78,6 +82,7 @@ DIFFERENT_DEPENDENCIES = [
         ("git+https://git.example.com/MyProject.git@master", ""),
         ("git+https://git.example.com/MyProject.git@v1.0", ""),
         ("git+https://git.example.com/MyProject.git@refs/pull/123/head", ""),
+        ("gopher:/foo/com", ""),
         (None, "==={ws}arbitrarystring"),
         (None, "({ws}==={ws}arbitrarystring{ws})"),
         (None, "=={ws}1.0"),
@@ -103,9 +108,9 @@ DIFFERENT_DEPENDENCIES = [
 @pytest.mark.parametrize("whitespace", ["", " ", "\t"])
 def test_basic_valid_requirement_parsing(
     name: str,
-    extras: Set[str],
+    extras: set[str],
     specifier: str,
-    url: Optional[str],
+    url: str | None,
     marker: str,
     whitespace: str,
 ) -> None:
@@ -164,6 +169,8 @@ class TestRequirementParsing:
         [
             "file:///absolute/path",
             "file://.",
+            "file:.",
+            "file:/.",
         ],
     )
     def test_file_url(self, url: str) -> None:
@@ -279,11 +286,48 @@ class TestRequirementParsing:
         # THEN
         assert ctx.exconly() == (
             "packaging.requirements.InvalidRequirement: "
-            "Expected closing RIGHT_PARENTHESIS\n"
+            "Expected matching RIGHT_PARENTHESIS for LEFT_PARENTHESIS, "
+            "after version specifier\n"
             "    name (>= 1.0\n"
             "         ~~~~~~~^"
         )
 
+    def test_error_when_prefix_match_is_used_incorrectly(self) -> None:
+        # GIVEN
+        to_parse = "black (>=20.*) ; extra == 'format'"
+
+        # WHEN
+        with pytest.raises(InvalidRequirement) as ctx:
+            Requirement(to_parse)
+
+        # THEN
+        assert ctx.exconly() == (
+            "packaging.requirements.InvalidRequirement: "
+            ".* suffix can only be used with `==` or `!=` operators\n"
+            "    black (>=20.*) ; extra == 'format'\n"
+            "           ~~~~~^"
+        )
+
+    @pytest.mark.parametrize("operator", [">=", "<=", ">", "<", "~="])
+    def test_error_when_local_version_label_is_used_incorrectly(
+        self, operator: str
+    ) -> None:
+        # GIVEN
+        to_parse = f"name {operator} 1.0+local.version.label"
+        op_tilde = len(operator) * "~"
+
+        # WHEN
+        with pytest.raises(InvalidRequirement) as ctx:
+            Requirement(to_parse)
+
+        # THEN
+        assert ctx.exconly() == (
+            "packaging.requirements.InvalidRequirement: "
+            "Local version label can only be used with `==` or `!=` operators\n"
+            f"    name {operator} 1.0+local.version.label\n"
+            f"         {op_tilde}~~~~^"
+        )
+
     def test_error_when_bracket_not_closed_correctly(self) -> None:
         # GIVEN
         to_parse = "name[bar, baz >= 1.0"
@@ -295,7 +339,8 @@ class TestRequirementParsing:
         # THEN
         assert ctx.exconly() == (
             "packaging.requirements.InvalidRequirement: "
-            "Expected closing RIGHT_BRACKET\n"
+            "Expected matching RIGHT_BRACKET for LEFT_BRACKET, "
+            "after extras\n"
             "    name[bar, baz >= 1.0\n"
             "        ~~~~~~~~~~^"
         )
@@ -311,7 +356,8 @@ class TestRequirementParsing:
         # THEN
         assert ctx.exconly() == (
             "packaging.requirements.InvalidRequirement: "
-            "Expected closing RIGHT_BRACKET\n"
+            "Expected matching RIGHT_BRACKET for LEFT_BRACKET, "
+            "after extras\n"
             "    name[bar, baz\n"
             "        ~~~~~~~~~^"
         )
@@ -332,6 +378,23 @@ class TestRequirementParsing:
             "           ~~~~~~~~~~~~~~~~~~~~~~^"
         )
 
+    def test_error_marker_bracket_unclosed(self) -> None:
+        # GIVEN
+        to_parse = "name; (extra == 'example'"
+
+        # WHEN
+        with pytest.raises(InvalidRequirement) as ctx:
+            Requirement(to_parse)
+
+        # THEN
+        assert ctx.exconly() == (
+            "packaging.requirements.InvalidRequirement: "
+            "Expected matching RIGHT_PARENTHESIS for LEFT_PARENTHESIS, "
+            "after marker expression\n"
+            "    name; (extra == 'example'\n"
+            "          ~~~~~~~~~~~~~~~~~~~^"
+        )
+
     def test_error_no_url_after_at(self) -> None:
         # GIVEN
         to_parse = "name @ "
@@ -447,25 +510,6 @@ class TestRequirementParsing:
             "                ^"
         )
 
-    @pytest.mark.parametrize(
-        "url",
-        [
-            "gopher:/foo/com",
-            "file:.",
-            "file:/.",
-        ],
-    )
-    def test_error_on_invalid_url(self, url: str) -> None:
-        # GIVEN
-        to_parse = f"name @ {url}"
-
-        # WHEN
-        with pytest.raises(InvalidRequirement) as ctx:
-            Requirement(to_parse)
-
-        # THEN
-        assert "Invalid URL" in ctx.exconly()
-
     def test_error_on_legacy_version_outside_triple_equals(self) -> None:
         # GIVEN
         to_parse = "name==1.0.org1"
@@ -592,13 +636,26 @@ class TestRequirementBehaviour:
 
     @pytest.mark.parametrize("dep1, dep2", EQUAL_DEPENDENCIES)
     def test_equal_reqs_equal_hashes(self, dep1: str, dep2: str) -> None:
-        """Requirement objects created from equivalent strings should be equal."""
+        """Requirement objects created from equal strings should be equal."""
         # GIVEN / WHEN
         req1, req2 = Requirement(dep1), Requirement(dep2)
 
         assert req1 == req2
         assert hash(req1) == hash(req2)
 
+    @pytest.mark.parametrize("dep1, dep2", EQUIVALENT_DEPENDENCIES)
+    def test_equivalent_reqs_equal_hashes_unequal_strings(
+        self, dep1: str, dep2: str
+    ) -> None:
+        """Requirement objects created from equivalent strings should be equal,
+        even though their string representation will not."""
+        # GIVEN / WHEN
+        req1, req2 = Requirement(dep1), Requirement(dep2)
+
+        assert req1 == req2
+        assert hash(req1) == hash(req2)
+        assert str(req1) != str(req2)
+
     @pytest.mark.parametrize("dep1, dep2", DIFFERENT_DEPENDENCIES)
     def test_different_reqs_different_hashes(self, dep1: str, dep2: str) -> None:
         """Requirement objects created from non-equivalent strings should differ."""
